@@ -26,14 +26,18 @@ from mintpy.utils import (
 
 ###############################  Usage  ################################
 REFERENCE = """reference:
-  Yunjun, Z., H. Fattahi, and F. Amelung (2019), Small baseline InSAR time series analysis: 
+  Yunjun, Z., Fattahi, H. and Amelung, F. (2019), Small baseline InSAR time series analysis:
   Unwrapping error correction and noise reduction, Computers & Geosciences, 133, 104331,
   doi:10.1016/j.cageo.2019.104331.
 
-  Chaussard, E., R. Bürgmann, H. Fattahi, R. M. Nadeau, T. Taira, C. W. Johnson, and I. Johanson
-  (2015), Potential for larger earthquakes in the East San Francisco Bay Area due to the direct 
+  Chaussard, E., Bürgmann, R., Fattahi, H., Nadeau, R. M., Taira, T., Johnson, C. W. and Johanson, I.
+  (2015), Potential for larger earthquakes in the East San Francisco Bay Area due to the direct
   connection between the Hayward and Calaveras Faults, Geophysical Research Letters, 42(8),
   2734-2741, doi:10.1002/2015GL063575.
+
+  Kang, Y., Lu, Z., Zhao, C., Xu, Y., Kim, J. W., & Gallegos, A. J. (2021).InSAR monitoring
+  of creeping landslides in mountainous regions: A case study in Eldorado National Forest,
+  California. Remote Sensing of Environment, 258, 112400. doi:10.1016/j.rse.2021.112400
 """
 
 TEMPLATE = get_template_content('modify_network')
@@ -58,7 +62,7 @@ def create_parser():
                         help='Do not update auxilary files, e.g.\n' +
                              'maskConnComp.h5 or avgSpatialCoh.h5 from ifgramStack.h5')
 
-    # 1
+    # 1. temp/perp baseline, num of conn., dates, pair index, etc.
     parser.add_argument('--max-tbase', dest='tempBaseMax',
                         type=float, help='max temporal baseline in days')
     parser.add_argument('--max-pbase', dest='perpBaseMax',
@@ -77,27 +81,33 @@ def create_parser():
     parser.add_argument('--end-date', '--max-date', dest='endDate',
                         help='remove/drop interferograms with date later than end-date in YYMMDD or YYYYMMDD format')
 
-    # 2. Coherence-based network
-    cohBased = parser.add_argument_group('Coherence-based Network',
-                                         'Drop/modify network based on spatial coherence')
+    # 2. coherence-based network
+    cohBased = parser.add_argument_group('Data-driven network modification', 'Drop/modify network based on data')
+    # 2.1 coherence-based
     cohBased.add_argument('--coherence-based', dest='coherenceBased', action='store_true',
-                          help='Enable coherence-based network modification')
+                          help='Enable coherence-based network modification (default: %(default)s).')
+    cohBased.add_argument('--min-coherence', dest='minCoherence', type=float, default=0.7,
+                          help='Minimum coherence value (default: %(default)s).')
+    # 2.2 area-ratio-based
+    cohBased.add_argument('--area-ratio-based', dest='areaRatioBased', action='store_true',
+                          help='Enable area ratio-based network modification (default: %(default)s).')
+    cohBased.add_argument('--min-area-ratio', dest='minAreaRatio', type=float, default=0.75,
+                          help='Minimum area ratio value (default: %(default)s).')
+    # common parameters
     cohBased.add_argument('--no-mst', dest='keepMinSpanTree', action='store_false',
                           help='Do not keep interferograms in Min Span Tree network based on inversed mean coherene')
     cohBased.add_argument('--mask', dest='maskFile', default='waterMask.h5',
-                          help='Mask file used to calculate the spatial coherence\n'
-                               'Default: waterMask.h5 or None')
-    cohBased.add_argument('--aoi-yx', dest='aoiYX', type=str,
-                          help='AOI in y0:y1,x0:x1 for coherence calculation')
-    cohBased.add_argument('--aoi-lalo', dest='aoiLALO', type=str,
-                          help='AOI in lat0:lat1,lon0:lon1 for coherence calculation')
-    cohBased.add_argument('--min-coherence', dest='minCoherence', type=float, default=0.7,
-                          help='Minimum coherence value')
+                          help='Mask file used to calculate the spatial coherence '
+                               '(default: waterMask.h5 or None)')
+    cohBased.add_argument('--aoi-yx', dest='aoi_pix_box', type=int, nargs=4, metavar=('X0', 'Y0', 'X1', 'Y1'), default=None,
+                          help='AOI in row/column range for coherence calculation (default: %(default)s).')
+    cohBased.add_argument('--aoi-lalo', dest='aoi_geo_box', type=float, nargs=4, metavar=('W', 'S', 'E', 'N'), default=None,
+                          help='AOI in lat/lon range for coherence calculation (default: %(default)s).')
     cohBased.add_argument('--lookup', dest='lookupFile',
                           help='Lookup table/mapping transformation file for geo/radar coordinate conversion.\n' +
                                'Needed for mask AOI in lalo')
 
-    # 3 Manually select network
+    # 3. manual selection
     manual = parser.add_argument_group('Manual Network', 'Manually select/drop/modify network')
     manual.add_argument('--manual', action='store_true',
                         help='display network to manually choose line/interferogram to remove')
@@ -108,8 +118,6 @@ def cmd_line_parse(iargs=None):
     parser = create_parser()
     inps = parser.parse_args(args=iargs)
 
-    inps.aoi_geo_box = None
-    inps.aoi_pix_box = None
     if not inps.lookupFile:
         inps.lookupFile = ut.get_lookup_file()
 
@@ -123,7 +131,7 @@ def cmd_line_parse(iargs=None):
     if inps.template_file:
         inps = read_template2inps(inps.template_file, inps)
     elif all(not i for i in [inps.referenceFile, inps.tempBaseMax, inps.perpBaseMax, inps.connNumMax,
-                             inps.excludeIfgIndex, inps.excludeDate, inps.coherenceBased,
+                             inps.excludeIfgIndex, inps.excludeDate, inps.coherenceBased, inps.areaRatioBased,
                              inps.startDate, inps.endDate, inps.reset, inps.manual]):
         msg = 'No input option found to remove interferogram, exit.\n'
         msg += 'To manually modify network, please use --manual option '
@@ -169,22 +177,22 @@ def read_template2inps(template_file, inps=None):
     keyList = [i for i in list(inpsDict.keys()) if prefix+i in template.keys()]
     for key in keyList:
         value = template[prefix+key]
-        if key in ['coherenceBased', 'keepMinSpanTree']:
+        if key in ['coherenceBased', 'areaRatioBased', 'keepMinSpanTree']:
             inpsDict[key] = value
         elif value:
-            if key in ['minCoherence', 'tempBaseMax', 'perpBaseMax']:
+            if key in ['minCoherence', 'minAreaRatio', 'tempBaseMax', 'perpBaseMax']:
                 inpsDict[key] = float(value)
             elif key in ['connNumMax']:
                 inpsDict[key] = int(value)
             elif key in ['maskFile', 'referenceFile']:
                 inpsDict[key] = value
             elif key == 'aoiYX':
-                tmp = [i.strip() for i in value.split(',')]
+                tmp = [i.replace('[','').replace(']','').strip() for i in value.split(',')]
                 sub_y = sorted([int(i.strip()) for i in tmp[0].split(':')])
                 sub_x = sorted([int(i.strip()) for i in tmp[1].split(':')])
                 inps.aoi_pix_box = (sub_x[0], sub_y[0], sub_x[1], sub_y[1])
             elif key == 'aoiLALO':
-                tmp = [i.strip() for i in value.split(',')]
+                tmp = [i.replace('[','').replace(']','').strip() for i in value.split(',')]
                 sub_lat = sorted([float(i.strip()) for i in tmp[0].split(':')])
                 sub_lon = sorted([float(i.strip()) for i in tmp[1].split(':')])
                 inps.aoi_geo_box = (sub_lon[0], sub_lat[1], sub_lon[1], sub_lat[0])
@@ -196,14 +204,16 @@ def read_template2inps(template_file, inps=None):
             elif key in ['startDate', 'endDate']:
                 inpsDict[key] = ptime.yyyymmdd(value)
             elif key == 'excludeDate':
-                inpsDict[key] = ptime.yyyymmdd(value.replace(',', ' ').split())
+                value = value.replace('[','').replace(']','').replace(',', ' ')
+                inpsDict[key] = ptime.yyyymmdd(value.split())
             elif key == 'excludeIfgIndex':
-                inpsDict[key] += value.replace(',', ' ').split()
-                inps.excludeIfgIndex = read_input_index_list(inps.excludeIfgIndex, stackFile=inps.file)
+                value = value.replace('[','').replace(']','').replace(',', ' ')
+                inpsDict[key] += value.split()
+                inpsDict[key] = read_input_index_list(inpsDict[key], stackFile=inps.file)
 
     # Turn reset on if 1) no input options found to drop ifgram AND 2) there is template input
     if all(not i for i in [inps.referenceFile, inps.tempBaseMax, inps.perpBaseMax, inps.connNumMax,
-                           inps.excludeIfgIndex, inps.excludeDate, inps.coherenceBased,
+                           inps.excludeIfgIndex, inps.excludeDate, inps.coherenceBased, inps.areaRatioBased,
                            inps.startDate, inps.endDate, inps.reset, inps.manual]):
         print('No input option found to remove interferogram')
         print('Keep all interferograms by enable --reset option')
@@ -291,6 +301,47 @@ def manual_select_pairs_to_remove(stackFile):
     return date12_click
 
 
+def get_aoi_pix_box(meta, lookup_file, pix_box, geo_box):
+    """Get pix_box for AOI."""
+    coord = ut.coordinate(meta, lookup_file=lookup_file)
+
+    # geo_box -> pix_box
+    if geo_box and lookup_file:
+        print('input AOI in (lon0, lat1, lon1, lat0): {}'.format(geo_box))
+        pix_box = coord.bbox_geo2radar(geo_box)
+
+    # check pix_box
+    if pix_box:
+        pix_box = coord.check_box_within_data_coverage(pix_box)
+        print('input AOI in (x0,y0,x1,y1): {}'.format(pix_box))
+
+    return pix_box
+
+
+def get_mst_date12(keep_mst, par_list_all, date12_list_all, date12_to_drop, min_par, par_name='average coherence'):
+    """Get the date12_list of the MST network for the given parameter."""
+    if keep_mst:
+        print(f'Get minimum spanning tree (MST) of interferograms with inverse of {par_name}.')
+        msg = ('Drop ifgrams with '
+               '1) {} < {} AND '
+               '2) not in MST network: '.format(par_name, min_par))
+
+        # get the current remaining network (after all the above criteria and before data-driven)
+        date12_to_keep = sorted(list(set(date12_list_all) - set(date12_to_drop)))
+        par_to_keep = [par for par, date12 in zip(par_list_all, date12_list_all)
+                       if date12 in date12_to_keep]
+
+        # get MST from the current remaining network
+        mst_date12_list = pnet.threshold_coherence_based_mst(date12_to_keep, par_to_keep)
+        mst_date12_list = ptime.yyyymmdd_date12(mst_date12_list)
+
+    else:
+        msg = 'Drop ifgrams with {} < {}: '.format(par_name, min_par)
+        mst_date12_list = []
+
+    return mst_date12_list, msg
+
+
 def get_date12_to_drop(inps):
     """Get date12 list to dropped
     Return [] if no ifgram to drop, thus KEEP ALL ifgrams;
@@ -315,45 +366,6 @@ def get_date12_to_drop(inps):
         date12_to_drop += tempList
         print('date12 not in reference file: ({})\n{}'.format(len(tempList), tempList))
 
-    # coherence file
-    if inps.coherenceBased:
-        print('--------------------------------------------------')
-        print('use coherence-based network modification')
-        coord = ut.coordinate(obj.metadata, lookup_file=inps.lookupFile)
-        if inps.aoi_geo_box and inps.lookupFile:
-            print('input AOI in (lon0, lat1, lon1, lat0): {}'.format(inps.aoi_geo_box))
-            inps.aoi_pix_box = coord.bbox_geo2radar(inps.aoi_geo_box)
-        if inps.aoi_pix_box:
-            inps.aoi_pix_box = coord.check_box_within_data_coverage(inps.aoi_pix_box)
-            print('input AOI in (x0,y0,x1,y1): {}'.format(inps.aoi_pix_box))
-
-        # Calculate spatial average coherence
-        cohList = ut.spatial_average(inps.file,
-                                     datasetName='coherence',
-                                     maskFile=inps.maskFile,
-                                     box=inps.aoi_pix_box,
-                                     saveList=True)[0]
-        coh_date12_list = list(np.array(date12ListAll)[np.array(cohList) >= inps.minCoherence])
-
-        # MST network
-        if inps.keepMinSpanTree:
-            print('Get minimum spanning tree (MST) of interferograms with inverse of coherence.')
-            msg = ('Drop ifgrams with '
-                   '1) average coherence < {} AND '
-                   '2) not in MST network: '.format(inps.minCoherence))
-            mst_date12_list = pnet.threshold_coherence_based_mst(date12ListAll, cohList)
-            mst_date12_list = ptime.yyyymmdd_date12(mst_date12_list)
-        else:
-            msg = 'Drop ifgrams with average coherence < {}: '.format(inps.minCoherence)
-            mst_date12_list = []
-
-        tempList = sorted(list(set(date12ListAll) - set(coh_date12_list + mst_date12_list)))
-        date12_to_drop += tempList
-        msg += '({})'.format(len(tempList))
-        if len(tempList) <= 200:
-            msg += '\n{}'.format(tempList)
-        print(msg)
-
     # temp baseline threshold
     if inps.tempBaseMax:
         tempIndex = np.abs(obj.tbaseIfgram) > inps.tempBaseMax
@@ -391,9 +403,8 @@ def get_date12_to_drop(inps):
         date12_to_drop += tempList
         print('--------------------------------------------------')
         print('Drop ifgrams with the following index number: {}'.format(len(tempList)))
-        for i in range(len(tempList)):
-            print('{} : {}'.format(i, tempList[i]))
-            #len(tempList), zip(inps.excludeIfgIndex, tempList)))
+        for i, date12 in enumerate(tempList):
+            print('{} : {}'.format(i, date12))
 
     # excludeDate
     if inps.excludeDate:
@@ -421,6 +432,80 @@ def get_date12_to_drop(inps):
         print('Drop ifgrams with date later than: {} ({})\n{}'.format(
             inps.endDate, len(tempList), tempList))
 
+    # coherence file
+    if inps.coherenceBased:
+        print('--------------------------------------------------')
+        print('use coherence-based network modification')
+
+        # get area of interest for coherence calculation
+        pix_box = get_aoi_pix_box(obj.metadata, inps.lookupFile, inps.aoi_pix_box, inps.aoi_geo_box)
+
+        # calculate spatial average coherence
+        cohList = ut.spatial_average(inps.file,
+                                     datasetName='coherence',
+                                     maskFile=inps.maskFile,
+                                     box=pix_box,
+                                     saveList=True)[0]
+
+        # get coherence-based network
+        coh_date12_list = list(np.array(date12ListAll)[np.array(cohList) >= inps.minCoherence])
+
+        # get MST network
+        mst_date12_list, msg = get_mst_date12(inps.keepMinSpanTree, cohList, date12ListAll, date12_to_drop,
+                                              min_par=inps.minCoherence,
+                                              par_name='average coherence')
+
+        # drop all dates (below cohh thresh AND not in MST)
+        tempList = sorted(list(set(date12ListAll) - set(coh_date12_list + mst_date12_list)))
+        date12_to_drop += tempList
+
+        msg += '({})'.format(len(tempList))
+        if len(tempList) <= 200:
+            msg += '\n{}'.format(tempList)
+        print(msg)
+
+    # area ratio file
+    if inps.areaRatioBased:
+        print('--------------------------------------------------')
+        print('use area-ratio-based network modification')
+
+        # get area of interest for coherence calculation
+        pix_box = get_aoi_pix_box(obj.metadata, inps.lookupFile, inps.aoi_pix_box, inps.aoi_geo_box)
+
+        # calculate average coherence in masked out areas as threshold
+        meanMaskCoh = np.nanmean(ut.spatial_average(inps.file,
+                                                    datasetName='coherence',
+                                                    maskFile=inps.maskFile,
+                                                    saveList=True,
+                                                    reverseMask=True)[0])
+        print(f'Average coherence of {inps.maskFile} reverse is {meanMaskCoh:.2f}')
+
+        # calculate area-ratio with pixels greater than meanMaskCoh
+        areaRatioList = ut.spatial_average(inps.file,
+                                           datasetName='coherence',
+                                           maskFile=inps.maskFile,
+                                           box=pix_box,
+                                           saveList=True,
+                                           checkAoi=True,
+                                           threshold=meanMaskCoh)[0]
+
+        # get area-ratio-based network
+        area_ratio_date12_list = list(np.array(date12ListAll)[np.array(areaRatioList) >= inps.minAreaRatio])
+
+        # get MST network
+        mst_date12_list, msg = get_mst_date12(inps.keepMinSpanTree, areaRatioList, date12ListAll, date12_to_drop,
+                                              min_par=inps.minAreaRatio,
+                                              par_name='coherent area ratio')
+
+        # drop all dates (below area-ratio thresh AND not in MST)
+        tempList = sorted(list(set(date12ListAll) - set(area_ratio_date12_list + mst_date12_list)))
+        date12_to_drop += tempList
+
+        msg += '({})'.format(len(tempList))
+        if len(tempList) <= 200:
+            msg += '\n{}'.format(tempList)
+        print(msg)
+
     # Manually drop pairs
     if inps.manual:
         tempList = manual_select_pairs_to_remove(inps.file)
@@ -430,6 +515,7 @@ def get_date12_to_drop(inps):
         print('date12 selected to remove: ({})\n{}'.format(len(tempList), tempList))
         date12_to_drop += tempList
 
+    ## summary
     # drop duplicate date12 and sort in order
     date12_to_drop = sorted(list(set(date12_to_drop)))
     date12_to_keep = sorted(list(set(date12ListAll) - set(date12_to_drop)))
@@ -437,12 +523,16 @@ def get_date12_to_drop(inps):
     print('number of interferograms to remove: {}'.format(len(date12_to_drop)))
     print('number of interferograms to keep  : {}'.format(len(date12_to_keep)))
 
+    # print list of date to drop
     date_to_keep = [d for date12 in date12_to_keep for d in date12.split('_')]
     date_to_keep = sorted(list(set(date_to_keep)))
     date_to_drop = sorted(list(set(dateList) - set(date_to_keep)))
     if len(date_to_drop) > 0:
         print('number of acquisitions to remove: {}\n{}'.format(len(date_to_drop), date_to_drop))
 
+    # checking:
+    # 1) no new date12 to drop against existing file
+    # 2) no date12 left after dropping
     date12ListKept = obj.get_date12_list(dropIfgram=True)
     date12ListDropped = sorted(list(set(date12ListAll) - set(date12ListKept)))
     if date12_to_drop == date12ListDropped:
@@ -450,6 +540,7 @@ def get_date12_to_drop(inps):
         date12_to_drop = None
     elif date12_to_drop == date12ListAll:
         raise Exception('Zero interferogram left! Please adjust your setting and try again.')
+
     return date12_to_drop
 
 
