@@ -10,15 +10,14 @@
 
 import os
 import csv
-import codecs
-from datetime import datetime as dt
+import glob
+import datetime as dt
 import numpy as np
 from pyproj import Geod
 from urllib.request import urlretrieve
 
-from mintpy.objects import timeseries
 from mintpy.objects.coord import coordinate
-from mintpy.utils import ptime, readfile, utils1 as ut
+from mintpy.utils import ptime, time_func, readfile, utils1 as ut
 
 
 unr_site_list_file = 'http://geodesy.unr.edu/NGLStationPages/DataHoldings.txt'
@@ -58,21 +57,21 @@ def search_gps(SNWE, start_date=None, end_date=None, site_list_file=None, min_nu
     site_names = txt_data[:, 0]
     site_lats, site_lons = txt_data[:, 1:3].astype(np.float32).T
     site_lons -= np.round(site_lons / (360.)) * 360.
-    t_start = np.array([dt.strptime(i, "%Y-%m-%d") for i in txt_data[:, 7].astype(str)])
-    t_end   = np.array([dt.strptime(i, "%Y-%m-%d") for i in txt_data[:, 8].astype(str)])
+    t0s = np.array([dt.datetime.strptime(i, "%Y-%m-%d") for i in txt_data[:, 7].astype(str)])
+    t1s = np.array([dt.datetime.strptime(i, "%Y-%m-%d") for i in txt_data[:, 8].astype(str)])
     num_solution = txt_data[:, 10].astype(np.int16)
 
-    # limit on space
+    # limit in space
     idx = ((site_lats >= SNWE[0]) * (site_lats <= SNWE[1]) *
            (site_lons >= SNWE[2]) * (site_lons <= SNWE[3]))
 
-    # limit on time
+    # limit in time
+    t0 = ptime.date_list2vector([start_date])[0][0] if start_date else None
+    t1 = ptime.date_list2vector([end_date])[0][0] if end_date else None
     if start_date:
-        t0 = ptime.date_list2vector([start_date])[0][0]
-        idx *= t_end >= t0
+        idx *= t1s >= t0
     if end_date:
-        t1 = ptime.date_list2vector([end_date])[0][0]
-        idx *= t_start <= t1
+        idx *= t0s <= t1
 
     # limit on number of solutions
     if min_num_solution is not None:
@@ -103,8 +102,8 @@ def get_baseline_change(dates1, pos_x1, pos_y1, pos_z1,
     return dates, bases
 
 
-def get_gps_los_obs(insar_file, site_names, start_date, end_date,
-                    gps_comp='enu2los', print_msg=True, redo=False):
+def get_gps_los_obs(insar_file, site_names, start_date, end_date, gps_comp='enu2los',
+                    horz_az_angle=-90., print_msg=True, redo=False):
     """Get the GPS LOS observations given the query info.
 
     Parameters: insar_file - str, InSAR LOS file, e.g. velocity or timeseries
@@ -113,6 +112,8 @@ def get_gps_los_obs(insar_file, site_names, start_date, end_date,
                 end_date   - str, date in YYYYMMDD format
                 gps_comp   - str, flag of projecting 2/3D GPS into LOS
                              e.g. enu2los, hz2los, up2los
+                horz_az_angle - float, azimuth angle of the horizontal motion in degree
+                             measured from the north with anti-clockwise as positive
                 print_msg  - bool, print verbose info
                 redo       - bool, ignore existing CSV file and re-calculate
     Returns:    site_obs   - 1D np.ndarray(), GPS LOS velocity or displacement in m or m/yr
@@ -125,18 +126,17 @@ def get_gps_los_obs(insar_file, site_names, start_date, end_date,
     fdir = os.path.dirname(insar_file)
     meta = readfile.read_attribute(insar_file)
     obs_type = meta['FILE_TYPE']
+    obs_ind = 4 if obs_type in ['velocity'] else 3
 
     # GPS CSV file info
-    csv_file = os.path.join(fdir, 'gps_{}.csv'.format(gps_comp))
-    col_names = ['Site', 'Lon', 'Lat', 'LOS_displacement']
-    col_names += ['LOS_velocity'] if obs_type == 'velocity' else []
-    num_col = len(col_names)
-    col_types = ['U10'] + ['f8'] * (num_col - 1)
+    csv_file = os.path.join(fdir, f'gps_{gps_comp}.csv')
+    col_names = ['Site', 'Lon', 'Lat', 'Displacement', 'Velocity']
+    col_types = ['U10'] + ['f8'] * (len(col_names) - 1)
 
     # skip re-calculate GPS if:
-    #    redo is False AND
-    #    csv_file exists (equivalent to num_row > 0) AND
-    #    num_row >= num_site
+    # 1. redo is False AND
+    # 2. csv_file exists (equivalent to num_row > 0) AND
+    # 3. num_row >= num_site
     num_row = 0
     if os.path.isfile(csv_file):
         fc = np.genfromtxt(csv_file, dtype=col_types, delimiter=',', names=True)
@@ -144,14 +144,14 @@ def get_gps_los_obs(insar_file, site_names, start_date, end_date,
 
     if not redo and os.path.isfile(csv_file) and num_row >= num_site:
         # read from existing CSV file
-        vprint('read GPS LOS observatioin from file: {}'.format(csv_file))
+        vprint('read GPS observations from file: {}'.format(csv_file))
         fc = np.genfromtxt(csv_file, dtype=col_types, delimiter=',', names=True)
-        site_obs = fc[col_names[-1]]
+        site_obs = fc[col_names[obs_ind]]
 
     else:
         # calculate and save to CSV file
         data_list = []
-        vprint('calculating GPS LOS observation ...')
+        vprint('calculating GPS observation ...')
 
         # get geom_obj (meta / geom_file)
         geom_file = ut.get_geometry_file(['incidenceAngle','azimuthAngle'], work_dir=fdir, coord='geo')
@@ -169,33 +169,36 @@ def get_gps_los_obs(insar_file, site_names, start_date, end_date,
 
             # calculate gps data value
             obj = GPS(site_name)
+            vel, dis_ts = obj.get_gps_los_velocity(
+                geom_obj,
+                start_date=start_date,
+                end_date=end_date,
+                gps_comp=gps_comp,
+                horz_az_angle=horz_az_angle)
 
-            if obs_type == 'velocity':
-                vel, dis_ts = obj.get_gps_los_velocity(geom_obj,
-                                                       start_date=start_date,
-                                                       end_date=end_date,
-                                                       gps_comp=gps_comp)
-                data = [dis_ts[-1] - dis_ts[0], vel] if dis_ts.size > 2 else [np.nan, np.nan]
-
-            elif obs_type == 'timeseries':
-                dis_ts = obj.read_gps_los_displacement(geom_obj,
-                                                       start_date=start_date,
-                                                       end_date=end_date,
-                                                       gps_comp=gps_comp)[1]
-                data = [dis_ts[-1] - dis_ts[0]] if dis_ts.size > 2 else [np.nan]
+            # ignore time-series if the estimated velocity is nan
+            dis = np.nan if np.isnan(vel) else dis_ts[-1] - dis_ts[0]
 
             # save data to list
-            data_list.append([obj.site, obj.site_lon, obj.site_lat] + data)
+            data_list.append([obj.site, obj.site_lon, obj.site_lat, dis, vel])
         prog_bar.close()
-        site_obs = np.array([x[-1] for x in data_list])
+
+        # # discard invalid sites
+        # flag = np.isnan([x[-1] for x in data_list])
+        # vprint('discard extra {} stations due to limited overlap/observations in time:'.format(np.sum(flag)))
+        # vprint('  {}'.format(np.array(data_list)[flag][:,0].tolist()))
+        # data_list = [x for x in data_list if not np.isnan(x[-1])]
 
         # write to CSV file
-        vprint('write GPS LOS observations to file: {}'.format(csv_file))
+        vprint('write GPS observations to file: {}'.format(csv_file))
         with open(csv_file, 'w') as fc:
             fcw = csv.writer(fc)
             fcw.writerow(col_names)
             fcw.writerows(data_list)
 
+        # prepare API output
+        site_obs = np.array([x[obs_ind] for x in data_list])
+
     return site_obs
 
 
@@ -203,12 +206,15 @@ def get_gps_los_obs(insar_file, site_names, start_date, end_date,
 
 #################################### Beginning of GPS-GSI utility functions ########################
 def read_pos_file(fname):
+    import codecs
     fcp = codecs.open(fname, encoding = 'cp1252')
     fc = np.loadtxt(fcp, skiprows=20, dtype=str, comments=('*','-DATA'))
 
-    dates = [dt(year=y, month=m, day=d) for y,m,d in zip(fc[:,0].astype(int),
-                                                         fc[:,1].astype(int),
-                                                         fc[:,2].astype(int))]
+    ys = fc[:,0].astype(int)
+    ms = fc[:,1].astype(int)
+    ds = fc[:,2].astype(int)
+    dates = [dt.datetime(year=y, month=m, day=d) for y,m,d in zip(ys, ms, ds)]
+
     X = fc[:,4].astype(np.float64).tolist()
     Y = fc[:,5].astype(np.float64).tolist()
     Z = fc[:,6].astype(np.float64).tolist()
@@ -241,8 +247,8 @@ def read_GSI_F3(gps_dir, site, start_date=None, end_date=None):
     Y = np.array(Y)
     Z = np.array(Z)
 
-    date0 = dt.strptime(start_date, "%Y%m%d")
-    date1 = dt.strptime(end_date, "%Y%m%d")
+    date0 = dt.datetime.strptime(start_date, "%Y%m%d")
+    date1 = dt.datetime.strptime(end_date, "%Y%m%d")
     flag = np.ones(X.shape, dtype=np.bool_)
     flag[dates < date0] = False
     flag[dates > date1] = False
@@ -274,7 +280,7 @@ class GPS:
 
     def __init__(self, site, data_dir='./GPS', version='IGS14'):
         self.site = site
-        self.data_dir = data_dir
+        self.data_dir = os.path.abspath(data_dir)
         self.version = version
         self.source = 'Nevada Geodetic Lab'
 
@@ -304,7 +310,12 @@ class GPS:
         self.plot_file_url = os.path.join(url_prefix, 'TimeSeries/{}.png'.format(site))
 
         # list of stations from Nevada Geodetic Lab
-        self.site_list_file = os.path.join(data_dir, 'DataHoldings.txt')
+        self.site_list_file = os.path.join(os.path.dirname(data_dir), 'DataHoldings.txt')
+        if not os.path.isfile(self.site_list_file):
+            dload_site_list()
+        site_names = np.loadtxt(self.site_list_file, dtype=bytes, skiprows=1, usecols=(0)).astype(str)
+        if site not in site_names:
+            raise ValueError('Site {} NOT found in file: {}'.format(site, unr_site_list_file))
 
         # directories for data files and plot files
         for fdir in [data_dir, os.path.dirname(self.plot_file)]:
@@ -361,7 +372,7 @@ class GPS:
             print('reading time and displacement in east/north/vertical direction')
         data = np.loadtxt(self.file, dtype=bytes, skiprows=1).astype(str)
 
-        self.dates = np.array([dt.strptime(i, "%y%b%d") for i in data[:, 1]])
+        self.dates = np.array([dt.datetime.strptime(i, "%y%b%d") for i in data[:, 1]])
         #self.dates = np.array([ptime.decimal_year2datetime(i) for i in data[:, 2]])
 
         (self.dis_e,
@@ -401,20 +412,23 @@ class GPS:
 
 
     #####################################  Utility Functions ###################################
-    def displacement_enu2los(self, inc_angle:float, head_angle:float, gps_comp='enu2los'):
+    def displacement_enu2los(self, inc_angle:float, az_angle:float, gps_comp='enu2los', horz_az_angle=-90.):
         """Convert displacement in ENU to LOS direction
-        Parameters: inc_angle  : float, local incidence angle in degree
-                    head_angle : float, satellite orbit heading direction in degree
+        Parameters: inc_angle : float, LOS incidence angle in degree
+                    az_angle  : float, LOS aziuth angle in degree
                         from the north, defined as positive in clock-wise direction
-                    gps_comp   : string, GPS components used to convert to LOS direction
-        Returns:    dis_los : 1D np.array for displacement in LOS direction
-                    std_los : 1D np.array for displacement standard deviation in LOS direction
+                    gps_comp  : string, GPS components used to convert to LOS direction
+                    horz_az_angle : float, fault azimuth angle used to convert horizontal to fault-parallel
+        Returns:    dis_los  : 1D np.array for displacement in LOS direction
+                    std_los  : 1D np.array for displacement standard deviation in LOS direction
         """
-        # get LOS unit vector
         inc_angle *= np.pi/180.
-        head_angle *= np.pi/180.
-        unit_vec = [np.sin(inc_angle) * np.cos(head_angle) * -1,
-                    np.sin(inc_angle) * np.sin(head_angle),
+        az_angle *= np.pi/180.
+        horz_az_angle *= np.pi/180.
+
+        # get LOS unit vector
+        unit_vec = [np.sin(inc_angle) * np.sin(az_angle) * -1,
+                    np.sin(inc_angle) * np.cos(az_angle),
                     np.cos(inc_angle)]
 
         gps_comp = gps_comp.lower()
@@ -425,22 +439,31 @@ class GPS:
         elif gps_comp in ['u2los', 'up2los']:
             unit_vec[0] = 0.
             unit_vec[1] = 0.
+        elif gps_comp in ['horz']:
+            unit_vec[0] = np.sin(horz_az_angle) * -1
+            unit_vec[1] = np.cos(horz_az_angle)
+            unit_vec[2] = 0.
+        elif gps_comp in ['vertical']:
+            unit_vec[0] = 0.
+            unit_vec[1] = 0.
+            unit_vec[2] = 1.
         else:
             raise ValueError('Un-known input gps components:'+str(gps_comp))
 
         # convert ENU to LOS direction
-        self.dis_los = (self.dis_e * unit_vec[0]
+        self.dis_los = (  self.dis_e * unit_vec[0]
                         + self.dis_n * unit_vec[1]
                         + self.dis_u * unit_vec[2])
-
         # assuming ENU component are independent with each other
-        self.std_los = ((self.std_e * unit_vec[0])**2
+        self.std_los = (   (self.std_e * unit_vec[0])**2
                          + (self.std_n * unit_vec[1])**2
-                         + (self.std_u * unit_vec[2])**2)**0.5
+                         + (self.std_u * unit_vec[2])**2 ) ** 0.5
+
         return self.dis_los, self.std_los
 
 
     def get_los_geometry(self, geom_obj, print_msg=False):
+        """Get the Line-of-Sight geometry info in incidence and azimuth angle in degrees."""
         lat, lon = self.get_stat_lat_lon(print_msg=print_msg)
 
         # get LOS geometry
@@ -454,31 +477,28 @@ class GPS:
             x = max(0, x);  x = min(int(atr['WIDTH'])-1, x)
             box = (x, y, x+1, y+1)
             inc_angle = readfile.read(geom_obj, datasetName='incidenceAngle', box=box, print_msg=print_msg)[0][0,0]
-            az_angle  = readfile.read(geom_obj, datasetName='azimuthAngle', box=box, print_msg=print_msg)[0][0,0]
-            head_angle = ut.azimuth2heading_angle(az_angle)
+            az_angle  = readfile.read(geom_obj, datasetName='azimuthAngle',   box=box, print_msg=print_msg)[0][0,0]
 
         elif isinstance(geom_obj, dict):
-            # use mean inc/head_angle from metadata
+            # use mean inc/az_angle from metadata
             inc_angle = ut.incidence_angle(geom_obj, dimension=0, print_msg=print_msg)
-            head_angle = float(geom_obj['HEADING'])
-            # for old reading of los.rdr band2 data into headingAngle directly
-            if (head_angle + 180.) > 45.:
-                head_angle = ut.azimuth2heading_angle(head_angle)
+            az_angle  = ut.heading2azimuth_angle(float(geom_obj['HEADING']))
 
         else:
             raise ValueError('input geom_obj is neight str nor dict: {}'.format(geom_obj))
 
-        return inc_angle, head_angle
+        return inc_angle, az_angle
 
 
     def read_gps_los_displacement(self, geom_obj, start_date=None, end_date=None, ref_site=None,
-                                  gps_comp:str='enu2los', print_msg=False):
+                                  gps_comp:str='enu2los', horz_az_angle=-90., print_msg=False):
         """Read GPS displacement in LOS direction
         Parameters: geom_obj : dict / str, metadata of InSAR file, or geometry file path
                     start_date : string in YYYYMMDD format
                     end_date   : string in YYYYMMDD format
                     ref_site   : string, reference GPS site
                     gps_comp   : string, GPS components used to convert to LOS direction
+                    az_angle   : float, fault azimuth angle used to convert horizontal to fault-parallel
         Returns:    dates : 1D np.array of datetime.datetime object
                     dis   : 1D np.array of displacement in meters
                     std   : 1D np.array of displacement uncertainty in meters
@@ -486,17 +506,17 @@ class GPS:
                     ref_site_lalo : tuple of 2 float, lat/lon of reference GPS site
         """
         # read GPS object
-        inc_angle, head_angle = self.get_los_geometry(geom_obj)
+        inc_angle, az_angle = self.get_los_geometry(geom_obj)
         dates = self.read_displacement(start_date, end_date, print_msg=print_msg)[0]
-        dis, std = self.displacement_enu2los(inc_angle, head_angle, gps_comp=gps_comp)
+        dis, std = self.displacement_enu2los(inc_angle, az_angle, gps_comp=gps_comp, horz_az_angle=horz_az_angle)
         site_lalo = self.get_stat_lat_lon(print_msg=print_msg)
 
         # get LOS displacement relative to another GPS site
         if ref_site:
             ref_obj = GPS(site=ref_site, data_dir=self.data_dir)
             ref_obj.read_displacement(start_date, end_date, print_msg=print_msg)
-            inc_angle, head_angle = ref_obj.get_los_geometry(geom_obj)
-            ref_obj.displacement_enu2los(inc_angle, head_angle, gps_comp=gps_comp)
+            inc_angle, az_angle = ref_obj.get_los_geometry(geom_obj)
+            ref_obj.displacement_enu2los(inc_angle, az_angle, gps_comp=gps_comp, horz_az_angle=horz_az_angle)
             ref_site_lalo = ref_obj.get_stat_lat_lon(print_msg=print_msg)
 
             # get relative LOS displacement on common dates
@@ -515,21 +535,36 @@ class GPS:
 
 
     def get_gps_los_velocity(self, geom_obj, start_date=None, end_date=None, ref_site=None,
-                             gps_comp='enu2los'):
-        dates, dis = self.read_gps_los_displacement(geom_obj,
-                                                    start_date=start_date,
-                                                    end_date=end_date,
-                                                    ref_site=ref_site,
-                                                    gps_comp=gps_comp)[:2]
+                             gps_comp='enu2los', horz_az_angle=-90.):
+
+        dates, dis = self.read_gps_los_displacement(
+            geom_obj,
+            start_date=start_date,
+            end_date=end_date,
+            ref_site=ref_site,
+            gps_comp=gps_comp,
+            horz_az_angle=horz_az_angle)[:2]
 
         # displacement -> velocity
-        date_list = [dt.strftime(i, '%Y%m%d') for i in dates]
-        if len(date_list) > 2:
-            A = timeseries.get_design_matrix4time_func(date_list)
+        # skip if:
+        # 1. num of observations <= 2 OR
+        # 2. time overlap < 1/4
+        flag_overlap = True
+        if start_date and end_date:
+            t0 = ptime.date_list2vector([start_date])[0][0]
+            t1 = ptime.date_list2vector([end_date])[0][0]
+            if dates[-1] - dates[0] < (t1 - t0) / 4:
+                flag_overlap = False
+
+        if len(dates) > 2 and flag_overlap:
+            date_list = [dt.datetime.strftime(i, '%Y%m%d') for i in dates]
+            A = time_func.get_design_matrix4time_func(date_list)
             self.velocity = np.dot(np.linalg.pinv(A), dis)[1]
+
         else:
             self.velocity = np.nan
 
         return self.velocity, dis
 
 #################################### End of GPS-UNR class ####################################
+
