@@ -111,7 +111,6 @@ def cmd_line_parse(iargs=None):
     parser = create_parser()
     inps = parser.parse_args(args=iargs)
 
-    
     atr = readfile.read_attribute(inps.file)
     if atr['FILE_TYPE'] != 'ifgramStack':
         # turn ON wirte_data for non-ifgramStack file by default
@@ -190,7 +189,12 @@ def reference_file(inps):
     # Check 1 - stack and its non-nan mask pixel coverage
     # outFile=False --> no avgPhaseVelocity file is generated due to the lack of reference point info.
     # did not use maskConnComp.h5 because not all input dataset has connectComponent info
-    stack = ut.temporal_average(inps.file, datasetName='unwrapPhase', updateMode=True, outFile=False)[0]
+    if atr['FILE_TYPE'] == 'ifgramStack':
+        ds_names = readfile.get_dataset_list(inps.file)
+        ds_name = [i for i in ds_names if i in ['unwrapPhase', 'rangeOffset', 'azimuthOffset']][0]
+    else:
+        ds_name = None
+    stack = ut.temporal_average(inps.file, datasetName=ds_name, updateMode=True, outFile=False)[0]
     mask = np.multiply(~np.isnan(stack), stack != 0.)
     if np.nansum(mask) == 0.0:
         raise ValueError('no pixel found with valid phase value in all datasets.')
@@ -302,12 +306,12 @@ def reference_point_attribute(atr, y, x):
 
 ###############################################################
 def manual_select_reference_yx(data, inps, mask=None):
+    """Manually select reference point in row/column number.
+    Parameters: data : 2D np.ndarray, stack of input file
+                inps : namespace, with key 'REF_X' and 'REF_Y', which will be updated
+                mask : 2D np.ndarray
     """
-    Input: 
-        data4display : 2D np.array, stack of input file
-        inps    : namespace, with key 'REF_X' and 'REF_Y', which will be updated
-    """
-    import matplotlib.pyplot as plt
+    from matplotlib import pyplot as plt
     print('\nManual select reference point ...')
     print('Click on a pixel that you want to choose as the refernce ')
     print('    pixel in the time-series analysis;')
