@@ -13,13 +13,12 @@ import os
 import sys
 import time
 import argparse
-import h5py
 import numpy as np
 from scipy import linalg
 
 from mintpy.defaults.template import get_template_content
 from mintpy.objects import timeseries, giantTimeseries, HDFEOS, cluster
-from mintpy.utils import arg_group, readfile, writefile, ptime, utils as ut
+from mintpy.utils import arg_group, ptime, time_func, readfile, writefile, utils as ut
 
 
 dataType = np.float32
@@ -77,9 +76,14 @@ def create_parser():
                                      formatter_class=argparse.RawTextHelpFormatter,
                                      epilog=TEMPLATE+'\n'+REFERENCE+'\n'+EXAMPLE)
 
+    # inputs
     parser.add_argument('timeseries_file',
                         help='Time series file for velocity inversion.')
     parser.add_argument('--template', '-t', dest='template_file', help='template file with options')
+    parser.add_argument('--ts-std-file', dest='ts_std_file',
+                        help='Time-series STD file for velocity STD calculation.')
+
+    # outputs
     parser.add_argument('-o', '--output', dest='outfile', help='output file name')
     parser.add_argument('--update', dest='update_mode', action='store_true',
                         help='Enable update mode, and skip estimation if:\n'+
@@ -87,11 +91,10 @@ def create_parser():
                              'and newer than input file\n' +
                              '2) all configuration parameters are the same.')
 
-    parser.add_argument('--ts-std-file', dest='ts_std_file',
-                        help='Time-series STD file for velocity STD calculation.')
-
     # reference in time and space
     # for input file without reference info, e.g. ERA5.h5
+    parser.add_argument('--ref-lalo', dest='ref_lalo', metavar=('LAT', 'LON'), type=float, nargs=2,
+                        help='Change referene point LAT LON for estimation.')
     parser.add_argument('--ref-yx', dest='ref_yx', metavar=('Y', 'X'), type=int, nargs=2,
                         help='Change referene point Y X for estimation.')
     parser.add_argument('--ref-date', dest='ref_date', metavar='DATE',
@@ -118,6 +121,13 @@ def create_parser():
     # time functions
     parser = arg_group.add_timefunc_argument(parser)
 
+    # residual file
+    resid = parser.add_argument_group('Residual file', 'Save residual displacement time-series to HDF5 file.')
+    resid.add_argument('--save-res', '--save_residual', dest='save_res', action='store_true',
+                       help='Save the residual displacement time-series to HDF5 file.')
+    resid.add_argument('--res-file', '--residual-file', dest='res_file', default='timeseriesResidual.h5',
+                       help='Output file name for the residual time-series file (default: %(default)s).')
+
     # computing
     parser = arg_group.add_memory_argument(parser)
 
@@ -149,6 +159,16 @@ def cmd_line_parse(iargs=None):
     # Initialize the dictionaries of exp and log funcs
     inps = init_exp_log_dicts(inps)
 
+    # --ref-lalo option
+    if inps.ref_lalo:
+        atr = readfile.read_attribute(inps.timeseries_file)
+        coord = ut.coordinate(atr)
+        ref_y, ref_x = coord.geo2radar(inps.ref_lalo[0], inps.ref_lalo[1])[:2]
+        if ref_y is not None and ref_x is not None:
+            inps.ref_yx = [ref_y, ref_x]
+            print('input reference point in (lat, lon): ({}, {})'.format(inps.ref_lalo[0], inps.ref_lalo[1]))
+            print('corresponding   point in (y, x): ({}, {})'.format(inps.ref_yx[0], inps.ref_yx[1]))
+
     return inps
 
 
@@ -339,12 +359,17 @@ def read_date_info(inps):
 
     # output file name
     if not inps.outfile:
+        fbase = os.path.splitext(os.path.basename(inps.timeseries_file))[0]
         outname = 'velocity'
         if inps.key == 'giantTimeseries':
             prefix = os.path.basename(inps.timeseries_file).split('PARAMS')[0]
             outname = prefix + outname
+        elif fbase in ['timeseriesRg', 'timeseriesAz']:
+            suffix = fbase.split('timeseries')[-1]
+            outname = outname + suffix
         outname += '.h5'
         inps.outfile = outname
+
     return inps
 
 
@@ -403,55 +428,6 @@ def read_inps2model(inps, date_list=None):
 
 
 ############################################################################
-def estimate_time_func(model, date_list, dis_ts, ref_date=None):
-    """
-    Deformation model estimator, using a suite of linear, periodic, step, exponential, and logarithmic function(s).
-
-    Gm = d
-
-    Parameters: date_list - list of str, dates in YYYYMMDD format
-                dis_ts    - 2D np.ndarray, displacement observation in size of (num_date, num_pixel)
-                model     - dict of time functions, e.g.:
-                            {'polynomial' : 2,                    # int, polynomial degree with 1 (linear), 2 (quadratic), 3 (cubic), etc.
-                             'periodic'   : [1.0, 0.5],           # list of float, period(s) in years. 1.0 (annual), 0.5 (semiannual), etc.
-                             'step'       : ['20061014'],         # list of str, date(s) in YYYYMMDD.
-                             'exp'        : {'20181026': [60],    # dict, key for onset time in YYYYMMDD and value for char. times in days.
-                                             ...
-                                            },
-                             'log'        : {'20161231': [80.5],  # dict, key for onset time in YYYYMMDD and value for char. times in days.
-                                             '20190125': [100, 200],
-                                             ...
-                                            },
-                             ...
-                             }
-    Returns:    G         - 2D np.ndarray, design matrix           in size of (num_date, num_param)
-                m         - 2D np.ndarray, parameter solution      in size of (num_param, num_pixel)
-                e2        - 1D np.ndarray, sum of squared residual in size of (num_pixel,)
-    """
-
-    G = timeseries.get_design_matrix4time_func(date_list, model, refDate=ref_date)
-
-    # least squares solver
-    # Opt. 1: m = np.linalg.pinv(G).dot(dis_ts)
-    # Opt. 2: m = scipy.linalg.lstsq(G, dis_ts, cond=1e-15)[0]
-    # Numpy is not used because it can not handle NaN value in dis_ts
-    m, e2 = linalg.lstsq(G, dis_ts, cond=None)[:2]
-
-    # check empty e2 due to the rank-deficient G matrix for sigularities.
-    e2 = np.array(e2)
-    if e2.size == 0:
-        print('\nWarning: empty e2 residues array due to a redundant or rank-deficient G matrix. This can cause sigularities.')
-        print('Please check: https://docs.scipy.org/doc/scipy/reference/generated/scipy.linalg.lstsq.html#scipy.linalg.lstsq')
-        print('The issue may be due to:')
-        print('\t1) very small char time(s), tau, of the exp/log function(s)')
-        print('\t2) the onset time(s) of exp/log are far earlier than the minimum date of the time series.')
-        print('Try a different char time, onset time.')
-        print('Your G matrix of the temporal model: \n', G)
-        raise ValueError('G matrix is redundant/rank-deficient!')
-
-    return G, m, e2
-
-
 def run_timeseries2time_func(inps):
 
     # basic info
@@ -459,6 +435,13 @@ def run_timeseries2time_func(inps):
     length, width = int(atr['LENGTH']), int(atr['WIDTH'])
     num_date = inps.numDate
     dates = np.array(inps.dateList)
+    seconds = atr.get('CENTER_LINE_UTC', 0)
+
+    # use the 1st date as reference if not found, e.g. timeseriesResidual.h5 file
+    if "REF_DATE" not in atr.keys() and not inps.ref_date:
+        inps.ref_date = inps.dateList[0]
+        print('WARNING: No REF_DATE found in time-series file or input in command line.')
+        print('  Set "--ref-date {}" and continue.'.format(inps.dateList[0]))
 
     # get deformation model from parsers
     model, num_param = read_inps2model(inps)
@@ -466,30 +449,40 @@ def run_timeseries2time_func(inps):
 
     ## output preparation
 
-    # attributes
-    atr['FILE_TYPE'] = 'velocity'
-    atr['UNIT'] = 'm/year'
-    atr['START_DATE'] = inps.dateList[0]
-    atr['END_DATE'] = inps.dateList[-1]
-    atr['DATE12'] = '{}_{}'.format(inps.dateList[0], inps.dateList[-1])
+    # time_func_param: attributes
+    atrV = dict(atr)
+    atrV['FILE_TYPE'] = 'velocity'
+    atrV['UNIT'] = 'm/year'
+    atrV['START_DATE'] = inps.dateList[0]
+    atrV['END_DATE'] = inps.dateList[-1]
+    atrV['DATE12'] = '{}_{}'.format(inps.dateList[0], inps.dateList[-1])
     if inps.ref_yx:
-        atr['REF_Y'] = inps.ref_yx[0]
-        atr['REF_X'] = inps.ref_yx[1]
+        atrV['REF_Y'] = inps.ref_yx[0]
+        atrV['REF_X'] = inps.ref_yx[1]
     if inps.ref_date:
-        atr['REF_DATE'] = inps.ref_date
+        atrV['REF_DATE'] = inps.ref_date
 
-    # config parameter
+    # time_func_param: config parameter
     print('add/update the following configuration metadata:\n{}'.format(configKeys))
     for key in configKeys:
-        atr[key_prefix+key] = str(vars(inps)[key])
+        atrV[key_prefix+key] = str(vars(inps)[key])
 
-    # instantiate output file
+    # time_func_param: instantiate output file
     ds_name_dict, ds_unit_dict = model2hdf5_dataset(model, ds_shape=(length, width))[1:]
     writefile.layout_hdf5(inps.outfile,
-                          metadata=atr,
+                          metadata=atrV,
                           ds_name_dict=ds_name_dict,
                           ds_unit_dict=ds_unit_dict)
 
+    # timeseries_res: attributes + instantiate output file
+    if inps.save_res:
+        atrR = dict(atr)
+        for key in ['REF_DATE']:
+            if key in atrR.keys():
+                atrR.pop(key)
+        writefile.layout_hdf5(inps.res_file, metadata=atrR, ref_file=inps.timeseries_file)
+
+
     ## estimation
 
     # calc number of box based on memory limit
@@ -504,13 +497,13 @@ def run_timeseries2time_func(inps):
 
     # loop for block-by-block IO
     for i, box in enumerate(box_list):
-        box_width  = box[2] - box[0]
-        box_length = box[3] - box[1]
-        num_pixel = box_length * box_width
+        box_wid  = box[2] - box[0]
+        box_len = box[3] - box[1]
+        num_pixel = box_len * box_wid
         if num_box > 1:
             print('\n------- processing patch {} out of {} --------------'.format(i+1, num_box))
-            print('box width:  {}'.format(box_width))
-            print('box length: {}'.format(box_length))
+            print('box width:  {}'.format(box_wid))
+            print('box length: {}'.format(box_len))
 
         # initiate output
         m = np.zeros((num_param, num_pixel), dtype=dataType)
@@ -534,7 +527,7 @@ def run_timeseries2time_func(inps):
             ts_data -= np.tile(ref_val.reshape(ts_data.shape[0], 1, 1), (1, ts_data.shape[1], ts_data.shape[2]))
 
         ts_data = ts_data[inps.dropDate, :, :].reshape(inps.numDate, -1)
-        if atr['UNIT'] == 'mm':
+        if atrV['UNIT'] == 'mm':
             ts_data *= 1./1000.
 
         ts_std = None
@@ -570,40 +563,36 @@ def run_timeseries2time_func(inps):
 
 
         ### estimation / solve Gm = d
+        print('estimating time functions via linalg.lstsq ...')
 
         if inps.bootstrap:
             ## option 1 - least squares with bootstrapping
             # Bootstrapping is a resampling method which can be used to estimate properties
             # of an estimator. The method relies on independently sampling the data set with
             # replacement.
-
-            try:
-                from sklearn.utils import resample
-            except ImportError:
-                raise ImportError('can not import scikit-learn!')
-            print('using bootstrap resampling {} times ...'.format(inps.bootstrapCount))
+            print('estimating time function STD with bootstrap resampling ({} times) ...'.format(inps.bootstrapCount))
 
             # calc model of all bootstrap sampling
+            rng = np.random.default_rng()
             m_boot = np.zeros((inps.bootstrapCount, num_param, num_pixel2inv), dtype=dataType)
             prog_bar = ptime.progressBar(maxValue=inps.bootstrapCount)
             for i in range(inps.bootstrapCount):
                 # bootstrap resampling
-                boot_ind = resample(np.arange(inps.numDate),
-                                    replace=True,
-                                    n_samples=inps.numDate)
+                boot_ind = rng.choice(inps.numDate, size=inps.numDate, replace=True)
                 boot_ind.sort()
 
                 # estimation
-                m_boot[i] = estimate_time_func(model=model,
-                                               date_list=dates[boot_ind].tolist(),
-                                               dis_ts=ts_data[boot_ind])[1]
+                m_boot[i] = time_func.estimate_time_func(
+                    model=model,
+                    date_list=dates[boot_ind].tolist(),
+                    dis_ts=ts_data[boot_ind],
+                    seconds=seconds)[1]
 
                 prog_bar.update(i+1, suffix='iteration {} / {}'.format(i+1, inps.bootstrapCount))
             prog_bar.close()
-            del ts_data
+            #del ts_data
 
             # get mean/std among all bootstrap sampling
-            print('calculate mean and standard deviation of bootstrap estimations')
             m[:, mask] = m_boot.mean(axis=0).reshape(num_param, -1)
             m_std[:, mask] = m_boot.std(axis=0).reshape(num_param, -1)
             del m_boot
@@ -611,12 +600,12 @@ def run_timeseries2time_func(inps):
 
         else:
             ## option 2 - least squares with uncertainty propagation
-
-            print('estimate time functions via linalg.lstsq ...')
-            G, m[:, mask], e2 = estimate_time_func(model=model,
-                                                   date_list=inps.dateList,
-                                                   dis_ts=ts_data)
-            del ts_data
+            G, m[:, mask], e2 = time_func.estimate_time_func(
+                model=model,
+                date_list=inps.dateList,
+                dis_ts=ts_data,
+                seconds=seconds)
+            #del ts_data
 
             ## Compute the covariance matrix for model parameters: Gm = d
             # C_m_hat = (G.T * C_d^-1, * G)^-1  # linear propagation from the TS covariance matrix. (option 2.1)
@@ -645,6 +634,7 @@ def run_timeseries2time_func(inps):
 
             else:
                 # option 2.2a - assume obs errors following normal dist. in time
+                print('estimating time function STD from time-series fitting residual ...')
                 G_inv = linalg.inv(np.dot(G.T, G))
                 m_var = e2.reshape(1, -1) / (num_date - num_param)
                 m_std[:, mask] = np.sqrt(np.dot(np.diag(G_inv).reshape(-1, 1), m_var))
@@ -655,15 +645,25 @@ def run_timeseries2time_func(inps):
                 # t_diff = G[:, 1] - np.mean(G[:, 1])
                 # vel_std = np.sqrt(np.sum(ts_diff ** 2, axis=0) / np.sum(t_diff ** 2)  / (num_date - 2))
 
-        # write
+        # write - time func params
         block = [box[1], box[3], box[0], box[2]]
         ds_dict = model2hdf5_dataset(model, m, m_std, mask=mask)[0]
         for ds_name, data in ds_dict.items():
             writefile.write_hdf5_block(inps.outfile,
-                                       data=data.reshape(box_length, box_width),
+                                       data=data.reshape(box_len, box_wid),
                                        datasetName=ds_name,
                                        block=block)
 
+        # write - residual file
+        if inps.save_res:
+            block = [0, num_date, box[1], box[3], box[0], box[2]]
+            ts_res = np.ones((num_date, box_len*box_wid), dtype=np.float32) * np.nan
+            ts_res[:, mask] = ts_data - np.dot(G, m)[:, mask]
+            writefile.write_hdf5_block(inps.res_file,
+                                       data=ts_res.reshape(num_date, box_len, box_wid),
+                                       datasetName='timeseries',
+                                       block=block)
+
     return inps.outfile
 
 
