@@ -473,7 +473,7 @@ def get_delay_timeseries(inps, atr):
         temp_files =[fname for fname in [inps.dem_file,
                                          inps.inc_angle_file,
                                          inps.lat_file,
-                                         inps.lon_file] 
+                                         inps.lon_file]
                      if (fname is not None and 'pyaps' in fname)]
         if temp_files:
             print('delete temporary geometry files: {}'.format(temp_files))
@@ -483,18 +483,17 @@ def get_delay_timeseries(inps, atr):
     return
 
 
-def correct_timeseries(timeseries_file, trop_file, out_file):
+def correct_timeseries(dis_file, tropo_file, cor_dis_file):
+    # diff.py can handle different reference in space and time
+    # between the absolute tropospheric delay and the double referenced time-series
     print('\n------------------------------------------------------------------------------')
-    print('correcting delay for input time-series by calling diff.py')
-    cmd = 'diff.py {} {} -o {} --force'.format(timeseries_file,
-                                               trop_file,
-                                               out_file)
-    print(cmd)
-    status = subprocess.Popen(cmd, shell=True).wait()
-    if status != 0:
-        raise Exception(('Error while correcting timeseries file '
-                         'using diff.py with tropospheric delay file.'))
-    return out_file
+    print('correcting relative delay for input time-series using diff.py')
+    from mintpy import diff
+
+    iargs = [dis_file, tropo_file, '-o', cor_dis_file, '--force']
+    print('diff.py', ' '.join(iargs))
+    diff.main(iargs)
+    return cor_dis_file
 
 
 ###############################################################
