@@ -102,7 +102,7 @@ def read_pts2inps(inps, coord_obj):
                                                 inps.pts_yx[:, 1],
                                                 print_msg=False)[:2]
             inps.pts_lalo = np.array(inps.pts_lalo).T.reshape(-1, 2)
-        except:
+        except ValueError:
             pass
 
     return inps
@@ -212,12 +212,22 @@ def auto_figure_title(fname, datasetNames=[], inps_dict=None):
             fig_title = fbase
 
     elif fext in ['.h5','.he5']:
-        fig_title = fbase
+        # for generic HDF5 file, e.g. velocity, masks, horz/vert decomposed file, etc.
+        num_dset = len(readfile.get_dataset_list(fname))
+        if num_dset > 1 and len(datasetNames) == 1:
+            # for single subplot from a multi-dataset file
+            # keep meaningful suffix, e.g. geo_, sub_, etc.
+            fparts = os.path.basename(fname).rsplit('_', 1)
+            suffix = fparts[0] + '_' if len(fparts) > 1 else ''
+            fig_title = suffix + datasetNames[0]
+        else:
+            # for single subplot from a single-dataset file OR multi-subplots
+            fig_title = fbase
 
     else:
         fig_title = os.path.basename(fname)
         # show dataset name for multi-band binry files
-        num_band = int(atr.get('number_bands', '1'))
+        num_band = int(atr.get('BANDS', '1'))
         if num_band > 1 and len(datasetNames) == 1:
             fig_title += ' - {}'.format(datasetNames[0])
 
@@ -268,10 +278,11 @@ def auto_flip_direction(metadata, ax=None, print_msg=True):
     return flip_lr, flip_ud
 
 
-def auto_multilook_num(box, num_time, print_msg=True):
+def auto_multilook_num(box, num_time, max_memory=4.0, print_msg=True):
     """Calcualte the default/auto multilook number based on the input 3D shape.
     Parameters: box           - tuple of 4 int in (x0, y0, x1, y1) for the spatial bounding box
                 num_time      - int, the 3rd / time dimension size
+                max_memory    - float, max memory in GB
     Returns:    multilook_num - int, multilook number
     """
     # calc total number of pixels
@@ -279,12 +290,29 @@ def auto_multilook_num(box, num_time, print_msg=True):
 
     # calc auto multilook_num
     if   num_pixel > (64e6*320):  multilook_num = 32;      # 16k * 4k image with 320 subplots
+    elif num_pixel > (50e6*160):  multilook_num = 20;      # 10k * 5k image with 160 subplots
     elif num_pixel > (32e6*160):  multilook_num = 16;      #  8k * 4k image with 160 subplots
+    elif num_pixel > (18e6*160):  multilook_num = 12;      #  9k * 2k image with 160 subplots
     elif num_pixel > ( 8e6*160):  multilook_num = 8;       #  4k * 2k image with 160 subplots
+    elif num_pixel > ( 4e6*180):  multilook_num = 6;       #  2k * 2k image with 180 subplots
     elif num_pixel > ( 4e6*80) :  multilook_num = 4;       #  2k * 2k image with 80  subplots
+    elif num_pixel > ( 4e6*45) :  multilook_num = 3;       #  2k * 2k image with 45  subplots
     elif num_pixel > ( 4e6*20) :  multilook_num = 2;       #  2k * 2k image with 20  subplots
     else:                         multilook_num = 1;
 
+    ## scale based on memory
+    # The auto calculation above uses ~1.5 GB in reserved memory and ~700 MB in actual memory.
+    if max_memory <= 2.0:
+        # With a lower  max memory from manual input, we increase the multilook_num (lower resolution)
+        multilook_num *= np.sqrt(4.0 / max_memory)
+    elif max_memory <= 4.0:
+        # Do nothing if input max memory is between 2.0-4.0 GB.
+        pass
+    else:
+        # With a larger max memory from manual input, we decrease the multilook_num (higher resolution)
+        multilook_num /= np.sqrt(max_memory / 4.0)
+    multilook_num = int(np.ceil(multilook_num))
+
     # print out msg
     if multilook_num > 1 and print_msg:
         print('total number of pixels: {:.1E}'.format(num_pixel))
@@ -506,7 +534,7 @@ def plot_coherence_history(ax, date12List, cohList, p_dict={}):
                                 every_year=p_dict['every_year'])[0]
     ax.set_ylim([p_dict['vlim'][0], p_dict['vlim'][1]])
 
-    ax.set_xlabel('Time [years]', fontsize=p_dict['fontsize'])
+    #ax.set_xlabel('Time [years]', fontsize=p_dict['fontsize'])
     ax.set_ylabel(p_dict['ds_name'], fontsize=p_dict['fontsize'])
     ax.legend(loc='lower right')
 
@@ -542,7 +570,7 @@ def plot_network(ax, date12List, dateList, pbaseList, p_dict={}, date12List_drop
 
     # For colorful display of coherence
     if 'cohList'     not in p_dict.keys():  p_dict['cohList']     = None
-    if 'xlabel'      not in p_dict.keys():  p_dict['xlabel']      = 'Time [years]'
+    if 'xlabel'      not in p_dict.keys():  p_dict['xlabel']      = None #'Time [years]'
     if 'ylabel'      not in p_dict.keys():  p_dict['ylabel']      = 'Perp Baseline [m]'
     if 'cbar_label'  not in p_dict.keys():  p_dict['cbar_label']  = 'Average Spatial Coherence'
     if 'cbar_size'   not in p_dict.keys():  p_dict['cbar_size']   = '3%'
@@ -588,8 +616,6 @@ def plot_network(ax, date12List, dateList, pbaseList, p_dict={}, date12List_drop
 
     ## Keep/Drop - date12
     date12List_keep = sorted(list(set(date12List) - set(date12List_drop)))
-    idx_date12_keep = [date12List.index(i) for i in date12List_keep]
-    idx_date12_drop = [date12List.index(i) for i in date12List_drop]
     if not date12List_drop:
         p_dict['disp_drop'] = False
 
@@ -752,7 +778,7 @@ def plot_perp_baseline_hist(ax, dateList, pbaseList, p_dict={}, dateList_drop=[]
     ax = auto_adjust_xaxis_date(ax, datevector, fontsize=p_dict['fontsize'],
                                 every_year=p_dict['every_year'])[0]
     ax = auto_adjust_yaxis(ax, pbaseList, fontsize=p_dict['fontsize'])
-    ax.set_xlabel('Time [years]', fontsize=p_dict['fontsize'])
+    #ax.set_xlabel('Time [years]', fontsize=p_dict['fontsize'])
     ax.set_ylabel('Perpendicular Baseline [m]', fontsize=p_dict['fontsize'])
 
     return ax
@@ -984,6 +1010,20 @@ def prepare_dem_background(dem, inps=None, print_msg=True):
             print(('show contour in step of {} m '
                    'with smoothing factor of {}').format(inps.dem_contour_step,
                                                          inps.dem_contour_smooth))
+
+    # masking
+    if inps and inps.mask_dem and (dem_shade is not None or dem_contour is not None):
+        dem_shape = [x.shape[:2] for x in [dem_shade, dem_contour] if x is not None][0]
+        if inps.msk.shape == dem_shape:
+            if print_msg:
+                print('mask DEM to be consistent with valid data coverage')
+            if dem_shade is not None:
+                dem_shade[inps.msk == 0] = np.nan
+            if dem_contour is not None:
+                dem_contour[inps.msk == 0] = np.nan
+        else:
+            print('WARNING: DEM has different size than mask, ignore --mask-dem and continue.')
+
     return dem_shade, dem_contour, dem_contour_sequence
 
 
@@ -1070,7 +1110,6 @@ def plot_gps(ax, SNWE, inps, metadata=dict(), print_msg=True):
     from mintpy.objects import gps
     vprint = print if print_msg else lambda *args, **kwargs: None
 
-    marker_size = 7
     vmin, vmax = inps.vlim
     cmap = ColormapExt(inps.colormap).colormap if isinstance(inps.colormap, str) else inps.colormap
 
@@ -1078,18 +1117,33 @@ def plot_gps(ax, SNWE, inps, metadata=dict(), print_msg=True):
     atr['UNIT'] = 'm'
     unit_fac = scale_data2disp_unit(metadata=atr, disp_unit=inps.disp_unit)[2]
 
-    if not inps.gps_start_date:
-        inps.gps_start_date = metadata.get('START_DATE', None)
+    start_date = inps.gps_start_date if inps.gps_start_date else metadata.get('START_DATE', None)
+    end_date = inps.gps_end_date if inps.gps_end_date else metadata.get('END_DATE', None)
 
-    if not inps.gps_end_date:
-        inps.gps_end_date = metadata.get('END_DATE', None)
+    # query for GNSS stations
+    site_names, site_lats, site_lons = gps.search_gps(SNWE, start_date, end_date)
+    if site_names.size == 0:
+        raise ValueError('No GNSS found within {} during {} - {}!'.format(SNWE, start_date, end_date))
+
+    # mask out stations not coincident with InSAR data
+    if inps.mask_gps and inps.msk is not None:
+        msk = inps.msk if inps.msk.ndim == 2 else np.prod(inps.msk, axis=-1)
+        coord = coordinate(metadata)
+        site_ys, site_xs = coord.geo2radar(site_lats, site_lons)[0:2]
+        flag = msk[site_ys, site_xs] != 0
+        # update station list
+        site_names = site_names[flag]
+        site_lats = site_lats[flag]
+        site_lons = site_lons[flag]
+        # check
+        if site_names.size == 0:
+            raise ValueError('No GNSS left after --mask-gps!')
 
-    site_names, site_lats, site_lons = gps.search_gps(SNWE, inps.gps_start_date, inps.gps_end_date)
     if inps.ref_gps_site and inps.ref_gps_site not in site_names:
         raise ValueError('input reference GPS site "{}" not available!'.format(inps.ref_gps_site))
 
     k = metadata['FILE_TYPE']
-    if inps.gps_component and k not in ['velocity', 'timeseries']:
+    if inps.gps_component and k not in ['velocity', 'timeseries', 'displacement']:
         inps.gps_component = None
         vprint('WARNING: --gps-comp is not implemented for {} file yet, set --gps-comp = None and continue'.format(k))
 
@@ -1098,31 +1152,32 @@ def plot_gps(ax, SNWE, inps, metadata=dict(), print_msg=True):
         vprint('-'*30)
         msg = 'plotting GPS '
         msg += 'velocity' if k == 'velocity' else 'displacement'
-        msg += ' in LOS direction'
+        msg += ' in {} direction'.format(inps.gps_component)
+        msg += ' with respect to {} ...'.format(inps.ref_gps_site) if inps.ref_gps_site else ' ...'
         vprint(msg)
         vprint('number of available GPS stations: {}'.format(len(site_names)))
-        vprint('start date: {}'.format(inps.gps_start_date))
-        vprint('end   date: {}'.format(inps.gps_end_date))
+        vprint('start date: {}'.format(start_date))
+        vprint('end   date: {}'.format(start_date))
         vprint('components projection: {}'.format(inps.gps_component))
 
         # get GPS LOS observations
+        # save absolute value to support both spatially relative and absolute comparison
+        # without compromising the re-usability of the CSV file
         site_obs = gps.get_gps_los_obs(
             insar_file=inps.file,
             site_names=site_names,
-            start_date=inps.gps_start_date,
-            end_date=inps.gps_end_date,
+            start_date=start_date,
+            end_date=end_date,
             gps_comp=inps.gps_component,
+            horz_az_angle=inps.horz_az_angle,
             print_msg=print_msg,
-            redo=inps.gps_redo,
-        )
+            redo=inps.gps_redo)
 
         # reference GPS
         if inps.ref_gps_site:
-            vprint('referencing all GPS LOS observations to site: {}'.format(inps.ref_gps_site))
             ref_ind = site_names.tolist().index(inps.ref_gps_site)
             # plot label of the reference site
-            ax.annotate(site_names[ref_ind], xy=(site_lons[ref_ind], site_lats[ref_ind]),
-                        fontsize=inps.font_size)
+            #ax.annotate(site_names[ref_ind], xy=(site_lons[ref_ind], site_lats[ref_ind]), fontsize=inps.font_size)
             # update value
             ref_val = site_obs[ref_ind]
             if not np.isnan(ref_val):
@@ -1131,21 +1186,36 @@ def plot_gps(ax, SNWE, inps, metadata=dict(), print_msg=True):
         # scale to the same unit as InSAR
         site_obs *= unit_fac
 
+        # exclude sites
+        if inps.ex_gps_sites:
+            ex_flag = np.array([x in inps.ex_gps_sites for x in site_names], dtype=np.bool_)
+            if np.sum(ex_flag) > 0:
+                vprint('ignore the following specified stations:\n  {}'.format(site_names[ex_flag]))
+                site_names = site_names[~ex_flag]
+                site_lats = site_lats[~ex_flag]
+                site_lons = site_lons[~ex_flag]
+                site_obs = site_obs[~ex_flag]
+
+        nan_flag = np.isnan(site_obs)
+        if np.sum(nan_flag) > 0:
+            vprint('ignore the following {} stations due to limited overlap/observations in time'.format(np.sum(nan_flag)))
+            vprint('  {}'.format(site_names[nan_flag]))
+
         # plot
         for lat, lon, obs in zip(site_lats, site_lons, site_obs):
-            color = cmap( (obs - vmin) / (vmax - vmin) ) if obs else 'none'
-            ax.scatter(lon, lat, color=color, s=marker_size**2, edgecolors='k', zorder=10)
+            if not np.isnan(obs):
+                color = cmap( (obs - vmin) / (vmax - vmin) )
+                ax.scatter(lon, lat, color=color, s=inps.gps_marker_size**2, edgecolors='k', lw=0.5, zorder=10)
 
     else:
         # plot GPS locations only
         vprint('showing GPS locations')
-        ax.scatter(site_lons, site_lats, s=marker_size**2, color='w', edgecolors='k', zorder=10)
+        ax.scatter(site_lons, site_lats, s=inps.gps_marker_size**2, color='w', edgecolors='k', lw=0.5, zorder=10)
 
     # plot GPS label
     if inps.disp_gps_label:
-        for i in range(len(site_names)):
-            ax.annotate(site_names[i], xy=(site_lons[i], site_lats[i]),
-                        fontsize=inps.font_size)
+        for site_name, lat, lon in zip(site_names, site_lats, site_lons):
+            ax.annotate(site_name, xy=(lon, lat), fontsize=inps.font_size)
 
     return ax
 
@@ -1216,7 +1286,7 @@ def check_disp_unit_and_wrap(metadata, disp_unit=None, wrap=False, wrap_range=[-
 
     if wrap:
         # wrap is supported for displacement file types only
-        if disp_unit.split('/')[0] not in ['radian', 'm', 'cm', 'mm', '1']:
+        if disp_unit.split('/')[0] not in ['radian', 'm', 'cm', 'mm', '1', 'pixel']:
             wrap = False
             print('WARNING: re-wrap is disabled for disp_unit = {}'.format(disp_unit))
         elif disp_unit.split('/')[0] != 'radian' and (wrap_range[1] - wrap_range[0]) == 2.*np.pi:
@@ -1253,30 +1323,38 @@ def scale_data2disp_unit(data=None, metadata=dict(), disp_unit=None):
     # Calculate scaling factor  - 1
     # phase unit - length / angle
     if data_unit[0].endswith('m'):
-        if   disp_unit[0] == 'mm': scale *= 1000.0
-        elif disp_unit[0] == 'cm': scale *= 100.0
-        elif disp_unit[0] == 'dm': scale *= 10.0
-        elif disp_unit[0] == 'm' : scale *= 1.0
-        elif disp_unit[0] == 'km': scale *= 1/1000.0
+        if   disp_unit[0] == 'mm':  scale *= 1000.0
+        elif disp_unit[0] == 'cm':  scale *= 100.0
+        elif disp_unit[0] == 'dm':  scale *= 10.0
+        elif disp_unit[0] == 'm' :  scale *= 1.0
+        elif disp_unit[0] == 'km':  scale *= 0.001
+        elif disp_unit[0] in ['in','inch']:  scale *= 39.3701
+        elif disp_unit[0] in ['ft','foot']:  scale *= 3.28084
+        elif disp_unit[0] in ['yd','yard']:  scale *= 1.09361
+        elif disp_unit[0] in ['mi','mile']:  scale *= 0.000621371
         elif disp_unit[0] in ['radians','radian','rad','r']:
-            range2phase = -(4*np.pi) / float(metadata['WAVELENGTH'])
+            range2phase = -4. * np.pi / float(metadata['WAVELENGTH'])
             scale *= range2phase
         else:
             print('Unrecognized display phase/length unit:', disp_unit[0])
-            pass
 
-        if   data_unit[0] == 'mm': scale *= 0.001
-        elif data_unit[0] == 'cm': scale *= 0.01
-        elif data_unit[0] == 'dm': scale *= 0.1
-        elif data_unit[0] == 'km': scale *= 1000.
+        # if stored data unit is not meter
+        if   data_unit[0] == 'mm':  scale *= 0.001
+        elif data_unit[0] == 'cm':  scale *= 0.01
+        elif data_unit[0] == 'dm':  scale *= 0.1
+        elif data_unit[0] == 'km':  scale *= 1000.
 
     elif data_unit[0] == 'radian':
         phase2range = -float(metadata['WAVELENGTH']) / (4*np.pi)
-        if   disp_unit[0] == 'mm': scale *= phase2range * 1000.0
-        elif disp_unit[0] == 'cm': scale *= phase2range * 100.0
-        elif disp_unit[0] == 'dm': scale *= phase2range * 10.0
-        elif disp_unit[0] == 'm' : scale *= phase2range * 1.0
-        elif disp_unit[0] == 'km': scale *= phase2range * 1/1000.0
+        if   disp_unit[0] == 'mm':  scale *= phase2range * 1000.0
+        elif disp_unit[0] == 'cm':  scale *= phase2range * 100.0
+        elif disp_unit[0] == 'dm':  scale *= phase2range * 10.0
+        elif disp_unit[0] == 'm' :  scale *= phase2range * 1.0
+        elif disp_unit[0] == 'km':  scale *= phase2range * 1/1000.0
+        elif disp_unit[0] in ['in','inch']:  scale *= phase2range * 39.3701
+        elif disp_unit[0] in ['ft','foot']:  scale *= phase2range * 3.28084
+        elif disp_unit[0] in ['yd','yard']:  scale *= phase2range * 1.09361
+        elif disp_unit[0] in ['mi','mile']:  scale *= phase2range * 0.000621371
         elif disp_unit[0] in ['radians','radian','rad','r']:
             pass
         else:
@@ -1303,11 +1381,11 @@ def scale_data2disp_unit(data=None, metadata=dict(), disp_unit=None):
                 print('Un-scalable display unit:', disp_unit[0])
     else:
         print('Un-scalable data unit:', data_unit)
+        disp_unit = [metadata['UNIT']]
 
     # Calculate scaling factor  - 2
     if len(data_unit) == 2:
         try:
-            disp_unit[1]
             if   disp_unit[1] in ['y','yr','year'  ]: disp_unit[1] = 'year'
             elif disp_unit[1] in ['m','mon','month']: disp_unit[1] = 'mon'; scale *= 12.0
             elif disp_unit[1] in ['d','day'        ]: disp_unit[1] = 'day'; scale *= 365.25
@@ -1580,8 +1658,13 @@ def draw_scalebar(ax, geo_box, unit='degrees', loc=[0.2, 0.2, 0.1], labelpad=0.0
     ## length
     # 1. calc scene width in meters
     if unit.startswith('deg'):
-        scene_width = geod.inv(geo_box[0], geo_box[3],
-                               geo_box[2], geo_box[3])[2]
+        if (geo_box[2] - geo_box[0]) > 30:
+            # do not plot scalebar if the longitude span > 30 deg
+            scene_width = None
+            return ax
+        else:
+            scene_width = geod.inv(geo_box[0], geo_box[3],
+                                   geo_box[2], geo_box[3])[2]
     elif unit.startswith('meter'):
         scene_width = geo_box[2] - geo_box[0]
 
